//
//  CBAppDelegate.m
//  CocoaBeam
//
//  Created by Jake on 3/23/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "CBAppDelegate.h"
#import "CBBeamPlanView.h"
#import "CBBeamSectionView.h"
#import "CBBeamDebugView.h"
#import "CBIntensityCurveView.h"
#import "CBDictionaryTransformer.h"
#import "CBFixture.h"

@implementation CBAppDelegate

+ (void)initialize {
	CBDictionaryTransformer* transformer = [[[CBDictionaryTransformer alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:transformer forName:@"CBDictionaryTransformer"];
}

- (id)init {
	if (self = [super init]) {
		NSString* fixtureFile = [[NSBundle mainBundle] pathForResource:@"FixtureDatabase" ofType:@""];
		fixtureList = [[NSKeyedUnarchiver unarchiveObjectWithFile:fixtureFile] retain];
		
		#ifdef DEMO
		[fixtureList makeObjectsPerformSelector:@selector(makeReadOnly)];
		#endif
	}
	return self;
}

- (void)awakeFromNib {
	[fixtureListController setContent:fixtureList];
	
	[curveView bind: @"fixture"
	           toObject: fixtureListController
	           withKeyPath: @"selection.self"
	           options: nil];
	
	[planView bind: @"fixture"
	          toObject: fixtureListController
	          withKeyPath: @"selection.self"
	          options: nil];

	[sectionView bind: @"fixture"
	             toObject: fixtureListController
	             withKeyPath: @"selection.self"
	             options: nil];
	
	[debugView   bind: @"fixture"
	             toObject: fixtureListController
	             withKeyPath: @"selection.self"
	             options: nil];
			
	[fixtureListController addObserver:planView
		       forKeyPath: @"selection.beamAnglesChanged"
		        options: NSKeyValueObservingOptionNew
		        context: NULL];
	
	[fixtureListController addObserver:sectionView
	       forKeyPath: @"selection.beamAnglesChanged"
	        options: NSKeyValueObservingOptionNew
	        context: NULL];
	
	[fixtureListController addObserver:debugView
	       forKeyPath: @"selection.beamAnglesChanged"
	        options: NSKeyValueObservingOptionNew
	        context: NULL];

	[fixtureListController addObserver:curveView
	       forKeyPath: @"selection.beamAnglesChanged"
	        options: NSKeyValueObservingOptionNew
	        context: NULL];

	[geometryController addObserver:planView
		       forKeyPath: @"selection.geometryChanged"
		        options: NSKeyValueObservingOptionNew
		        context: NULL];

	[geometryController addObserver:sectionView
	       forKeyPath: @"selection.geometryChanged"
	        options: NSKeyValueObservingOptionNew
	        context: NULL];
	
	[geometryController addObserver:debugView
	       forKeyPath: @"selection.geometryChanged"
	        options: NSKeyValueObservingOptionNew
	        context: NULL];	
	
	
	#ifndef DEMO
		[inspector orderFront:self];
		NSLog(@"[%@ %s] FULL VERSION MODE", [self class], _cmd);
	#endif
	
	#ifdef DEMO
		NSImage* fullVersionImage = [NSImage imageNamed:@"FullMenuItem.png"];
		[inspectorMenu setAction:nil];
		[inspectorMenu setImage:fullVersionImage];
	#endif

}

- (IBAction) scaleChanged:(id)sender {
	[planView setScale:[sender floatValue]];
	[sectionView setScale:[sender floatValue]];
	[debugView setScale:[sender floatValue]];
}

- (IBAction) redraw:(id)sender {
	[planView redraw:sender];
	[sectionView redraw:sender];
}

- (IBAction) saveDocumentAs:(id)sender {
	NSSavePanel *save = [NSSavePanel savePanel];
	int result = [save runModal];
	if (result == NSOKButton){
		NSString *selectedFile = [save filename];
		[NSKeyedArchiver archiveRootObject:fixtureList toFile:selectedFile];
	}
}

- (IBAction) openDocument:(id)sender {
	NSOpenPanel *open = [NSOpenPanel openPanel];
	int result = [open runModal];
	if (result == NSOKButton){
		NSString *selectedFile = [open filename];
		//if (fixtureList) [fixtureList release];
		fixtureList = [[NSKeyedUnarchiver unarchiveObjectWithFile:selectedFile] retain];
		NSLog(@"[%@ %s] %@", [self class], _cmd, fixtureList);
		[fixtureListController setContent:fixtureList];
	}
	
}

@end
