//
//  CBBeamSectionView.m
//  CocoaBeam
//
//  Created by Jake on 3/23/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "CBBeamDebugView.h"
#import "CBFixture.h"
#import "CBGeometricModel.h"

#define SQUARED(x) (x*x)

@implementation CBBeamDebugView : NSView

- (id)initWithFrame:(NSRect)frame {
    if (self = [super initWithFrame:frame]) {
		scale=1.0;
    }
    return self;
}

- (void)drawRect:(NSRect)rect {
	const float WALL = -500.0;
	const float CEILING = 500.0;
	
	//Background is black
	[[NSColor blackColor] set];
	NSRectFill(rect);

	//Size
	NSSize size = [self bounds].size;
	
	//This is the transform
	NSAffineTransform* xform = [NSAffineTransform transform];
	[xform translateXBy:(size.width/2) - ([model floorDistance]/2) yBy:size.height/4];
	[xform scaleBy:scale];
	[xform concat];
	
	float Xoffset = ([fixture angleXForIntensity:intensity] / 2);	
	float Yoffset = ([fixture angleYForIntensity:intensity] / 2);	
	float nearAngle = [model angleOfIncidence] - Xoffset;
	float farAngle = [model angleOfIncidence] + Xoffset;
	float aoi = [model angleOfIncidence];

	float floorDistanceAtBeamDistance = beamDistance * sin(RADIANS(aoi));
	float beamHeightAtBeamDistance = [model trimHeight] - (beamDistance * cos(RADIANS(aoi)));
	float normalDistanceToFloor = beamHeightAtBeamDistance / sin(RADIANS(aoi));
	float floorPointRightAngleAtBeamDistance = (beamHeightAtBeamDistance / tan(RADIANS(aoi)));
	float beamWidthAtDistance = beamDistance * tan(RADIANS(Xoffset));
	float beamHeightAtDistance = beamDistance * tan(RADIANS(Yoffset));
	float squareRootClause = 1-(SQUARED(normalDistanceToFloor/beamHeightAtDistance));
	NSPoint beamDistancePoint = NSMakePoint([model floorDistance]-floorDistanceAtBeamDistance,beamHeightAtBeamDistance);
	NSPoint rightAngleFloorPoint = NSMakePoint(beamDistancePoint.x+floorPointRightAngleAtBeamDistance,0.0);

	NSLog(@"[%@ %s] Intensity %f FloorPoint:%f", [self class], _cmd, intensity, [model floorPointForAngleOffset:-Xoffset]);

	if (squareRootClause > 0) {
		[[NSColor redColor] set];
		NSBezierPath* p = [NSBezierPath bezierPath];
		[p moveToPoint:NSMakePoint(rightAngleFloorPoint.x,0)];
		[p lineToPoint:NSMakePoint(rightAngleFloorPoint.x,-sqrt(squareRootClause)*beamWidthAtDistance)];
		[p stroke];
	}

	[[NSColor orangeColor] setFill];
	[[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(beamDistancePoint.x-1,beamDistancePoint.y-1,2,2)] fill];

	[[NSColor blueColor] setFill];
	[[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(rightAngleFloorPoint.x-0.5,-0.5,1,1)] fill];
	
	NSPoint nearPoint, farPoint;
	if (nearAngle < 90) {
		nearPoint = NSMakePoint([model floorPointForAngleOffset:-Xoffset], 0.0);
	} else {
		float distanceToWall = -WALL + [model floorDistance];
		float heightOnWall = (tan(RADIANS((nearAngle-90)))*distanceToWall) + [model trimHeight];
		nearPoint = NSMakePoint(WALL, heightOnWall);
	}

	//line to far point
	if (farAngle < 90 && farAngle) {
    	farPoint = NSMakePoint([model floorPointForAngleOffset:Xoffset], 0.0);			
	} else if (farAngle > 90 && farAngle < 180){
		float distanceToWall = -WALL + [model floorDistance];
		float heightOnWall = tan(RADIANS((farAngle-90)))*distanceToWall+[model trimHeight];
		farPoint = NSMakePoint(WALL,heightOnWall);
	} else if (farAngle > 180) {
		float distanceToCeiling = CEILING - [model trimHeight];
		float positionOnCeiling = [model floorDistance] + (distanceToCeiling * tan(RADIANS((farAngle-180))));
		farPoint = NSMakePoint(positionOnCeiling,CEILING);
	}
	
	NSPoint originPoint = NSMakePoint(0.0,0.0);
	NSPoint focusPoint = NSMakePoint(0.0,[model focusHeight]);
	NSPoint fixturePoint = NSMakePoint([model floorDistance], [model trimHeight]);
	NSPoint fixtureFloorPoint = NSMakePoint([model floorDistance], 0.0);
	
	NSBezierPath* path = [NSBezierPath bezierPath];
	//main triangle
	[path moveToPoint:originPoint];
	[path lineToPoint:focusPoint];
	[path lineToPoint:fixturePoint];
	[path lineToPoint:fixtureFloorPoint];
	[[NSColor whiteColor] set];
	[path stroke];
	
	//near/far point beam
	[path removeAllPoints];
	[path moveToPoint:nearPoint];
	[path lineToPoint:fixturePoint];
	[path lineToPoint:farPoint];
	[path setLineWidth:0.25];
	[[NSColor grayColor] set];
	[path stroke];
	
 	//draw origin
	if (1) {
		[[NSColor redColor] set];
		NSBezierPath* cross = [NSBezierPath bezierPath];
		[cross setLineWidth:0.5];
		float crossSize=5;
	    [cross moveToPoint:NSMakePoint(0.0,crossSize)];
	    [cross lineToPoint:NSMakePoint(0.0,-crossSize)];
	    [cross moveToPoint:NSMakePoint(crossSize,0.0)];
	    [cross lineToPoint:NSMakePoint(-crossSize,0.0)];
	    [cross stroke];
	}
	
	//draw fixture
	if (1) {
		[[NSColor blueColor] set];
		NSBezierPath* circle = [NSBezierPath bezierPath];
		float radius=5.0;
		[circle setLineWidth:0.5];
	    [circle appendBezierPathWithOvalInRect:NSMakeRect(([model floorDistance])-radius,([model trimHeight])-radius,2*radius,2*radius)];
	    [circle stroke];
	} 
	
	//draw label
	if (1) {
		[xform invert];
		[xform concat];
		NSFont *font = [NSFont fontWithName:@"Chalkboard" size:16.0];
		NSColor* color=[NSColor grayColor];
		NSDictionary *attrsDictionary = [NSDictionary dictionaryWithObjectsAndKeys:font,NSFontAttributeName,color,NSForegroundColorAttributeName,nil];
		NSString* string = [NSString stringWithString:@"Debug View"];
		[string drawAtPoint:NSMakePoint(10.0,10.0) withAttributes:attrsDictionary];
	}
	

}

- (float)scale {
	return scale;
}

- (void)setScale:(float)newScale{
	scale = newScale;
	[self display];
}

- (CBFixture*)fixture {
	return fixture;
}

- (void)setFixture:(CBFixture*)aValue {
	CBFixture* oldFixture = fixture;
	fixture = [aValue retain];
	[oldFixture release];
	[self display];
}

- (void)observeValueForKeyPath:(NSString *)keyPath
              ofObject:(id)object 
                        change:(NSDictionary *)change
                       context:(void *)context {
	[self display];
}

- (float)intensity {
	return intensity;
}

- (void)setIntensity:(float)newIntensity {
	intensity = newIntensity;
	[self display];
}

- (IBAction)setIntensityAction:(id)intensityObject {
	[self setIntensity: [intensityObject floatValue]];
}

- (IBAction)setBeamDistance:(id)beamDistanceObject {
	beamDistance = [beamDistanceObject floatValue];
	[self display];
}

- (IBAction)redraw:(id)sender {
	[self display];
}

@end
