//
//  CBBeamGradientView.m
//  CocoaBeam
//
//  Created by Jake on 3/21/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//


#import "CBBeamPlanView.h"
#import "CBGeometricModel.h"
#import "CBFixture.h"
#include "math.h"
//#include "CBIntensityCurve.h"

#define RADIANS(degrees) degrees/(180/pi)
#define DEGREES(radians) radians*(180/pi)
#define BEAM_WIDTH(distance,angle) 2*distance*tan((angle/2)/(180/pi))
#define SQUARED(x) (x*x)

@implementation CBBeamPlanView

- (id)initWithFrame:(NSRect)frame {
    if (self = [super initWithFrame:frame]) {
		scale=1.0;
    }
    return self;
}

- (void) updateBeamImage {
	//Release the old image and recreate a new one based on the size of the view
	[beamImage release];
	NSSize size = [self bounds].size;
	beamImage = [[NSImage alloc] initWithSize:size]; 
	
	//Begin drawing operations in the new NSImage
	[beamImage lockFocus];	

	//Composite copy, we want transparency, but don't want to layer it right now.
	[[NSGraphicsContext currentContext] setCompositingOperation: NSCompositeCopy];
	
	//Transform, set the origin to the center of the image
	NSAffineTransform* xform = [NSAffineTransform transform];
	[xform translateXBy:(size.width/2) yBy:size.height/2];
	[xform concat];

	//First, do the parabolic sections on the floor.
	float intensity;		
	for (intensity=0.0; intensity < 1.0; intensity+=0.05) {	
		float xangle = [fixture angleXForIntensity:(float)intensity];
		float yangle = [fixture angleYForIntensity:(float)intensity];
		
		float farAngle = (xangle/2) + [model angleOfIncidence];
		float nearAngle = [model angleOfIncidence] - (xangle/2);

		float x = [model floorPointForAngleOffset:xangle/2];
		float w = [model floorPointForAngleOffset:-xangle/2]-x;
		//float y = -[model floorWidthForAngle:yangle]/2; //not used
		float h = [model floorWidthForAngle:yangle];

		if (farAngle >= 90 && nearAngle < 90) {
			//ellipse
			float dist,c;
			NSBezierPath *curve = nil;
			//NSBezierPath *inverseCurve = nil;
			float step=0.1;
			for (c = 500; c >= -500; c-=step) {
				//step=abs(c)<40?0.01:20;
				dist = fabs(c);
				w = dist * tan(RADIANS((xangle/2)));
				h = dist * tan(RADIANS((yangle/2)));
				float aoi=[model angleOfIncidence];
				float floorThrow = dist * sin(RADIANS(aoi));
				float beamHeightAtDist = [model trimHeight] - (dist * cos(RADIANS(aoi)));
				float normalDistanceToFloor = beamHeightAtDist / sin(RADIANS(aoi));
				float squareRootClause = 1-(SQUARED(normalDistanceToFloor/fabs(h)));
				
				//We go negative, so stop.
				if (squareRootClause > 0) {
					float elipsePoint = sqrt(squareRootClause)*fabs(w);
					float sign = c>0?1.0:-1.0;
					if (!curve) {
						curve = [[NSBezierPath alloc] init];
						//inverseCurve = [[NSBezierPath alloc] init];
						[curve moveToPoint:NSMakePoint(([model floorDistance]-floorThrow)*scale,0)];
						[curve lineToPoint:NSMakePoint(([model floorDistance]-floorThrow)*scale,(sign*elipsePoint)*scale)];
						//[inverseCurve moveToPoint:NSMakePoint([model floorDistance]-floorThrow,0)];
						//[inverseCurve lineToPoint:NSMakePoint([model floorDistance]-floorThrow,-elipsePoint)];
					} else
						[curve lineToPoint:NSMakePoint((([model floorDistance]-floorThrow)*scale),(sign*elipsePoint)*scale)];
						//[inverseCurve lineToPoint:NSMakePoint([model floorDistance]-floorThrow,-elipsePoint)];
				}
			}
			[curve closePath];
			//[curve linearGradientFill:[NSColor colorWithCalibratedRed:intensity green:intensity blue:intensity alpha:1.0]];
			NSColor* color = [NSColor colorWithCalibratedRed:1.0 green:1.0 blue:1.0 alpha:intensity];
			[color setFill];
			[curve fill];
		}
	}
	
	//Next, do the elliptical sections
	for (intensity=0.0; intensity < 1.0; intensity+=0.05) {	
		float xangle = [fixture angleXForIntensity:(float)intensity];
		float yangle = [fixture angleYForIntensity:(float)intensity];
		
		float farAngle = (xangle/2) + [model angleOfIncidence];
		float nearAngle = (xangle/2) - [model angleOfIncidence];

		float x = [model floorPointForAngleOffset:xangle/2];
		float w = [model floorPointForAngleOffset:-xangle/2]-x;
		float y = -[model floorWidthForAngle:yangle]/2;
		float h = [model floorWidthForAngle:yangle];

		if (farAngle < 90 && nearAngle < 90) {
			NSRect region = NSMakeRect(x*scale,y*scale,w*scale,h*scale);
			[[NSColor colorWithCalibratedRed:1.0 green:1.0 blue:1.0 alpha:intensity] setFill];		
			[[NSBezierPath bezierPathWithOvalInRect:region] fill];
		} 
	}

	//Done drawing
	[beamImage unlockFocus];
}

- (NSImage*)beamImage {
	return beamImage;
}

- (void)drawRect:(NSRect)rect {
	//Fill the background black
	[[NSColor blackColor] set];
	NSRectFill(rect);

	//Origin is in the center of the view
	NSAffineTransform* xform = [NSAffineTransform transform];
	[xform translateXBy:NSWidth(rect)/2 yBy:NSHeight(rect)/2];
	[xform concat];
	
	//Draw the beam, note that the beam is (0,0) referenced, so we need to "undo" the transform
	[[self beamImage] compositeToPoint:NSMakePoint(-NSWidth(rect)/2,-NSHeight(rect)/2) operation:NSCompositeSourceOver];
	
	
	//draw desired area
	if (1) {
		[[NSColor yellowColor] set];
		NSSize areaSize = [model desiredArea];
		
		//Scaled appropriately.
		areaSize.width *= scale;
		areaSize.height *= scale;
		NSBezierPath* area = [NSBezierPath bezierPathWithRect:NSMakeRect(-areaSize.width/2,-areaSize.height/2,areaSize.width,areaSize.height)];
		float array[2];
		array[0] = 5.0; //segment painted with stroke color
		array[1] = 2.0; //segment not painted with a color
		[area setLineDash:array count:2 phase:0.0];
		[area stroke];
	}

 	//draw origin
	if (1) {
		[[NSColor redColor] set];
		NSBezierPath* cross = [NSBezierPath bezierPath];
		[cross setLineWidth:1.0];
		float crossSize=5;
	    [cross moveToPoint:NSMakePoint(0.0,crossSize)];
	    [cross lineToPoint:NSMakePoint(0.0,-crossSize)];
	    [cross moveToPoint:NSMakePoint(crossSize,0.0)];
	    [cross lineToPoint:NSMakePoint(-crossSize,0.0)];
	    [cross stroke];
	}
		
	//draw fixture
	if (1) {
		[[NSColor blueColor] set];
		NSBezierPath* circle = [NSBezierPath bezierPath];
		float radius=5.0;
		[circle setLineWidth:1.0];
	    [circle appendBezierPathWithOvalInRect:NSMakeRect(([model floorDistance]*scale)-radius,-radius,2*radius,2*radius)];
	    [circle stroke];
	}

	//draw Y field angle
	if (1) {
		[[NSColor colorWithCalibratedRed:0.1 green:0.1 blue:0.1 alpha:1.0] set];
		NSPoint fixturePt = NSMakePoint([model floorDistance]*scale,0);
		NSBezierPath* beam = [NSBezierPath bezierPath];
		[beam setLineWidth:1.0];
		float angle = [fixture fieldAngleY];
		[beam moveToPoint:NSMakePoint(0,([model floorDistance]*tan(RADIANS(angle/2)))*scale)];
		[beam lineToPoint:fixturePt];
		[beam lineToPoint:NSMakePoint(0,-([model floorDistance]*tan(RADIANS(angle/2)))*scale)];
		[beam stroke];
		
		[beam removeAllPoints];
		[[NSColor colorWithCalibratedRed:0.5 green:0.5 blue:0.5 alpha:1.0] set];
		angle = [fixture beamAngleY];
		[beam moveToPoint:NSMakePoint(0,([model floorDistance]*tan(RADIANS(angle/2)))*scale)];
		[beam lineToPoint:fixturePt];
		[beam lineToPoint:NSMakePoint(0,-([model floorDistance]*tan(RADIANS(angle/2))*scale))];
		[beam stroke];
	}

	//draw label
	if (1) {
		[xform invert];	//undo the transform
		[xform concat];
		NSFont *font = [NSFont fontWithName:@"Chalkboard" size:16.0];
		NSColor* color=[NSColor grayColor];
		NSDictionary *attrsDictionary = [NSDictionary dictionaryWithObjectsAndKeys:font,NSFontAttributeName,color,NSForegroundColorAttributeName,nil];
		NSString* string = [NSString stringWithString:@"Plan View"];
		[string drawAtPoint:NSMakePoint(10.0,10.0) withAttributes:attrsDictionary];
	}

}

- (float)scale {
	return scale;
}

- (void)setScale:(float)newScale {
	scale = newScale;
	[self updateBeamImage];
	[self display];
}

- (IBAction)redraw:(id)sender {
	[self updateBeamImage];
	[self display];
}


- (CBFixture*)fixture {
	return fixture;
}

- (void)setFixture:(CBFixture*)aValue {
	CBFixture* oldFixture = fixture;
	fixture = [aValue retain];
	[oldFixture release];
	[self updateBeamImage];
	[self display];
}

- (void)observeValueForKeyPath:(NSString *)keyPath
              ofObject:(id)object 
                        change:(NSDictionary *)change
                       context:(void *)context {
	[self updateBeamImage];
	[self display];
}

- (void)dealloc {
	[super dealloc];
}

@end


