//
//  CBBeamSectionView.m
//  CocoaBeam
//
//  Created by Jake on 3/23/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "CBBeamSectionView.h"
#import "CBFixture.h"
#import "CBGeometricModel.h"

@implementation CBBeamSectionView

- (id)initWithFrame:(NSRect)frame {
    if (self = [super initWithFrame:frame]) {
		scale=1.0;
    }
    return self;
}

- (NSImage*)beamImage {
	return beamImage;
}

- (void) updateBeamImage {
	const float WALL = -500.0;
	const float CEILING = 500.0;
		
	[beamImage release];
	NSSize size = [self bounds].size;
	beamImage = [[NSImage alloc] initWithSize:size]; 
	
	[beamImage lockFocus];	
	
	NSAffineTransform* xform = [NSAffineTransform transform];
	[xform translateXBy:(size.width/2) - ([model floorDistance]/2) yBy:size.height/4];
	[xform concat];
	
/*	NSAffineTransform* xform = [NSAffineTransform transform];
	[xform translateXBy:[beamImage size].width/2 yBy:0.0];
	[xform concat];
*/				
	[[NSGraphicsContext currentContext] setCompositingOperation: NSCompositeCopy];
	
	NSPoint fixturePoint = NSMakePoint([model floorDistance] * scale, [model trimHeight] * scale);

	float intensity;
	for (intensity=0.0; intensity < 1.0; intensity+=0.05) {
		float offset = ([fixture angleXForIntensity:intensity] / 2);	
		float nearAngle = [model angleOfIncidence] - offset;
		float farAngle = [model angleOfIncidence] + offset;
		
		NSBezierPath* beam = [NSBezierPath bezierPath];

		//move to near point
		if (nearAngle < 90) {
	    	[beam moveToPoint:NSMakePoint([model floorPointForAngleOffset:-offset] * scale, 0.0)];			
		} else {
			float distanceToWall = -WALL + [model floorDistance];
			float heightOnWall = (tan(RADIANS((nearAngle-90)))*distanceToWall) + [model trimHeight];
			[beam moveToPoint:NSMakePoint(WALL * scale ,heightOnWall * scale)];
		}
		
	    [beam lineToPoint:fixturePoint];
	
		//line to far point
		if (farAngle < 90 && farAngle) {
	    	[beam lineToPoint:NSMakePoint([model floorPointForAngleOffset:offset] * scale, 0.0)];			
		} else if (farAngle > 90 && farAngle < 180){
			float distanceToWall = -WALL + [model floorDistance];
			float heightOnWall = tan(RADIANS((farAngle-90)))*distanceToWall+[model trimHeight];
			[beam lineToPoint:NSMakePoint(WALL * scale,heightOnWall * scale)];
			if (nearAngle<90)
				[beam lineToPoint:NSMakePoint(WALL * scale,0)];
		} else if (farAngle > 180) {
			float distanceToCeiling = CEILING - [model trimHeight];
			float positionOnCeiling = [model floorDistance] + (distanceToCeiling * tan(RADIANS((farAngle-180))));
			[beam lineToPoint:NSMakePoint(positionOnCeiling * scale,CEILING * scale)];
		}
		[beam closePath];
		[[NSColor colorWithCalibratedRed:1.0 green:1.0 blue:1.0 alpha:intensity] setFill];
		[beam fill];
	}
	
	[beamImage unlockFocus];
	
}

- (void)drawRect:(NSRect)rect {
	//Background is black
	[[NSColor blackColor] set];
	NSRectFill(rect);

	//Size
	NSSize size = [self bounds].size;
	
	//This is the transform
	NSAffineTransform* xform = [NSAffineTransform transform];
	[xform translateXBy:(size.width/2) - ([model floorDistance]/2) yBy:size.height/4];
	[xform concat];
		
	//Draw the beam, note that the beam is (0,0) referenced, so we need to "undo" the transform
	[[self beamImage] compositeToPoint:NSMakePoint(-((size.width/2) - ([model floorDistance]/2)), -size.height/4) operation:NSCompositeSourceOver];

	//draw desired area
	if (1) {
		[[NSColor yellowColor] set];
		NSSize areaSize = [model desiredArea];
		
		//Scaled appropriately.
		areaSize.width *= scale;
		areaSize.height *= scale;
		
		
		NSBezierPath* area = [NSBezierPath bezierPathWithRect:NSMakeRect(-areaSize.width/2, 0.0,areaSize.width,[model focusHeight]*scale)];
		float array[2];
		array[0] = 5.0; //segment painted with stroke color
		array[1] = 2.0; //segment not painted with a color
		[area setLineDash:array count:2 phase:0.0];
		[area stroke];
	}
	
 	//draw origin
	if (1) {
		[[NSColor redColor] set];
		NSBezierPath* cross = [NSBezierPath bezierPath];
		[cross setLineWidth:1.0];
		float crossSize=5;
	    [cross moveToPoint:NSMakePoint(0.0,crossSize)];
	    [cross lineToPoint:NSMakePoint(0.0,-crossSize)];
	    [cross moveToPoint:NSMakePoint(crossSize,0.0)];
	    [cross lineToPoint:NSMakePoint(-crossSize,0.0)];
	    [cross stroke];
	}
	
	//draw fixture
	if (1) {
		[[NSColor blueColor] set];
		NSBezierPath* circle = [NSBezierPath bezierPath];
		float radius=5.0;
		[circle setLineWidth:1.0];
	    [circle appendBezierPathWithOvalInRect:NSMakeRect(([model floorDistance]*scale)-radius,([model trimHeight]*scale)-radius,2*radius,2*radius)];
	    [circle stroke];
	} 
	
	//draw label
	if (1) {
		[xform invert];
		[xform concat];
		NSFont *font = [NSFont fontWithName:@"Chalkboard" size:16.0];
		NSColor* color=[NSColor grayColor];
		NSDictionary *attrsDictionary = [NSDictionary dictionaryWithObjectsAndKeys:font,NSFontAttributeName,color,NSForegroundColorAttributeName,nil];
		NSString* string = [NSString stringWithString:@"Section View"];
		[string drawAtPoint:NSMakePoint(10.0,10.0) withAttributes:attrsDictionary];
	}
	

}

- (float)scale {
	return scale;
}

- (void)setScale:(float)newScale{
	scale = newScale;
	[self updateBeamImage];
	[self display];
}

- (CBFixture*)fixture {
	return fixture;
}

- (void)setFixture:(CBFixture*)aValue {
	CBFixture* oldFixture = fixture;
	fixture = [aValue retain];
	[oldFixture release];
	[self updateBeamImage];
	[self display];
}

- (void)observeValueForKeyPath:(NSString *)keyPath
              ofObject:(id)object 
                        change:(NSDictionary *)change
                       context:(void *)context {
	[self updateBeamImage];
	[self display];
}

- (IBAction)redraw:(id)sender {
	[self updateBeamImage];
	[self display];
}

@end
