//
//  CBFixture.h
//  CocoaBeam
//
//  Created by Jake on 3/23/07.
//  Copyright 2007 Jacob Bordens. All rights reserved.
//

#import <Cocoa/Cocoa.h>

//BeamShape keys
#define CBBeamShape @"Beam Shape"
#define CBCircularBeam @"Circle"
#define CBOvalBeam @"Oval"

//Circular, non-zooming
#define CBBeamAngle  @"Beam Angle"
#define CBFieldAngle @"Field Angle"

//Circular, zooming
#define CBFloodBeamAngle  @"Flood Beam Angle"
#define CBFloodFieldAngle @"Flood Field Angle"
#define CBSpotBeamAngle  @"Spot Beam Angle"
#define CBSpotFieldAngle @"Spot Field Angle"

//Oval non-zooming
#define CBBeamAngleX  @"Beam Angle (X-axis)"
#define CBFieldAngleX @"Field Angle (X-axis)"
#define CBBeamAngleY  @"Beam Angle (Y-axis)"
#define CBFieldAngleY @"Field Angle (Y-axis)"

//Oval zooming
#define CBFloodBeamAngleX  @"Flood Beam Angle (X-axis)"
#define CBFloodFieldAngleX @"Flood Field Angle (X-axis)"
#define CBFloodBeamAngleY  @"Flood Beam Angle (Y-axis)"
#define CBFloodFieldAngleY @"Flood Field Angle (Y-axis)"
#define CBSpotBeamAngleX  @"Spot Beam Angle (X-axis)"
#define CBSpotFieldAngleX @"Spot Field Angle (X-axis)"
#define CBSpotBeamAngleY  @"Spot Beam Angle (Y-axis)"
#define CBSpotFieldAngleY @"Spot Field Angle (Y-axis)"


@class CBIntensityCurve;
@class CBGeometricModel;

@interface CBFixture : NSObject <NSCoding> {
	//Fixture Properties
	NSDictionary* properties;

	//Zoom Level
	float zoom;
	
	//Intensity Curves
	CBIntensityCurve* intensityCurveX;
 	CBIntensityCurve* intensityCurveY;
}

//Properties
- (NSMutableDictionary*)properties;
- (void)setProperties:(NSDictionary*)aValue;
- (NSMutableDictionary*)beamAngles;
- (void)setBeamAngles:(NSDictionary*)newAngles;

//- (NSRect)beamRegionForIntensity:(float)intensity forGeometry:(CBGeometricModel*)model;
- (float)angleXForIntensity:(float)intensity;
- (float)angleYForIntensity:(float)intensity;

//Return beam/field angles based on current beam properties and zoom level
- (float)beamAngleX;
- (float)fieldAngleX;
- (float)beamAngleY;
- (float)fieldAngleY;

//Zoomable
- (float)zoom;
- (void)setZoom:(float)newZoom;

//Beam
- (NSString*)beamShape;
- (void)setBeamShape:(NSString*)newBeamShape;
- (NSArray*)beamShapes;
- (BOOL)isZoomable;
- (void)setIsZoomable:(BOOL)zoomable;

//For demo versions
- (void)makeReadOnly;
@end
