//
//  CBFixture.m
//  CocoaBeam
//
//  Created by Jake on 3/23/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "CBFixture.h"
#import "CBIntensityCurve.h"
#import "CBGeometricModel.h"

#define IIF(x,y) x?x:y

@interface CBFixture (Private)
	- (void)initializeCurves;
@end

@implementation CBFixture

+ (void) initialize
{
    NSArray *keys;
    keys = [NSArray arrayWithObjects: @"beamAngleX", @"beamAngleY", @"fieldAngleX", @"fieldAngleY", @"beamAngles", @"beamShape", @"isZoomable", @"zoom", nil];

    [CBFixture setKeys: keys
        triggerChangeNotificationsForDependentKey: @"beamAnglesChanged"];

} 

- (BOOL) beamAnglesChanged {
    return (YES);
}

- (id)init {
	if (self = [super init]){
		intensityCurveX = [[CBIntensityCurve alloc] init];
		intensityCurveY = [[CBIntensityCurve alloc] init];
		properties = [[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithFloat:20.0], CBBeamAngle,
																	    [NSNumber numberWithFloat:30.0], CBFieldAngle,
																	    [NSNumber numberWithBool:NO],@"isZoomable",
																	    CBCircularBeam, CBBeamShape,
																	    @"Default Fixture", @"name", nil] retain];																	
		[self initializeCurves];		
	}
	return self;
}

- (void)initializeCurves {
	//Initialize the curves
	NSArray* arrayX = [NSArray arrayWithObjects:[NSValue valueWithPoint:NSMakePoint(0,1.0)],
											    [NSValue valueWithPoint:NSMakePoint([self beamAngleX],0.5)],
											    [NSValue valueWithPoint:NSMakePoint([self fieldAngleX],0.1)],nil];
											
	[intensityCurveX setPoints:arrayX];

	NSArray* arrayY = [NSArray arrayWithObjects:[NSValue valueWithPoint:NSMakePoint(0,1.0)],
											    [NSValue valueWithPoint:NSMakePoint([self beamAngleY],0.5)],
											    [NSValue valueWithPoint:NSMakePoint([self fieldAngleY],0.1)],nil];

	[intensityCurveY setPoints:arrayY];
}

- (float)angleXForIntensity:(float)intensity {
	return [intensityCurveX angleForIntensity:intensity];
}

- (float)angleYForIntensity:(float)intensity {
	return [intensityCurveY angleForIntensity:intensity];
}

/*- (NSRect)beamRegionForIntensity:(float)intensity forGeometry:(CBGeometricModel*)model {
	float xangle = [intensityCurveX valueForY:intensity];
	float yangle = [intensityCurveY valueForY:intensity];
	float x, y, w, h;
	
	x = [model floorPointForAngleOffset:xangle/2];
	w = [model floorPointForAngleOffset:-xangle/2]-x;
	y = -[model floorWidthForAngle:yangle]/2;
	h = [model floorWidthForAngle:yangle];

	return NSMakeRect(x,y,w,h);
}
*/

- (NSString*)beamShape {
	return [properties objectForKey:CBBeamShape];
}

- (void)setBeamShape:(NSString*)newBeamShape {
	[self willChangeValueForKey:@"beamAngles"];
	[self willChangeValueForKey:@"beamAngleX"];
	[self willChangeValueForKey:@"beamAngleY"];
	[self willChangeValueForKey:@"fieldAngleX"];
	[self willChangeValueForKey:@"fieldAngleY"];
	[properties setObject:newBeamShape forKey:CBBeamShape];
	[self didChangeValueForKey:@"beamAngleX"];
	[self didChangeValueForKey:@"beamAngleY"];
	[self didChangeValueForKey:@"fieldAngleX"];
	[self didChangeValueForKey:@"fieldAngleY"];
	[self didChangeValueForKey:@"beamAngles"];
}

- (NSArray*)beamShapes{
	return [NSArray arrayWithObjects:CBCircularBeam, CBOvalBeam, nil];
}

- (BOOL)isZoomable {
	return [[properties objectForKey:@"isZoomable"] boolValue];
}
- (float)beamAngleX {
	if ([self isZoomable]) {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			float flood = [[properties objectForKey:CBFloodBeamAngle] floatValue];
			float spot = [[properties objectForKey:CBSpotBeamAngle] floatValue];
			return spot + ((flood-spot) * zoom);
		} else {
			float flood =  [[properties objectForKey:CBFloodBeamAngleX] floatValue];
			float spot  =  [[properties objectForKey:CBSpotBeamAngleX] floatValue];
			return spot + ((flood-spot) * zoom);
		} 
	} else {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			return [[properties objectForKey:CBBeamAngle] floatValue];
		} else {
			return [[properties objectForKey:CBBeamAngleX] floatValue];
		}
	}
}

- (float)fieldAngleX {
	if ([self isZoomable]) {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			float flood = [[properties objectForKey:CBFloodFieldAngle] floatValue];
			float spot = [[properties objectForKey:CBSpotFieldAngle] floatValue];
			return spot + ((flood-spot) * zoom);
		} else {
			float flood =  [[properties objectForKey:CBFloodFieldAngleX] floatValue];
			float spot  =  [[properties objectForKey:CBSpotFieldAngleX] floatValue];
			return spot + ((flood-spot) * zoom);
		}
	} else {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			return [[properties objectForKey:CBFieldAngle] floatValue];
		} else {
			return [[properties objectForKey:CBFieldAngleX] floatValue];
		}
	}
}

- (float)fieldAngleY {
	if ([self isZoomable]) {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			float flood = [[properties objectForKey:CBFloodFieldAngle] floatValue];
			float spot = [[properties objectForKey:CBSpotFieldAngle] floatValue];
			return spot + ((flood-spot) * zoom);
		} else {
			float flood =  [[properties objectForKey:CBFloodFieldAngleY] floatValue];
			float spot  =  [[properties objectForKey:CBSpotFieldAngleY] floatValue];
			return spot + ((flood-spot) * zoom);
		}
	} else {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			return [[properties objectForKey:CBFieldAngle] floatValue];
		} else {
			return [[properties objectForKey:CBFieldAngleY] floatValue];
		}
	}
}

- (float)beamAngleY {
	if ([self isZoomable]) {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			float flood = [[properties objectForKey:CBFloodBeamAngle] floatValue];
			float spot = [[properties objectForKey:CBSpotBeamAngle] floatValue];
			return spot + ((flood-spot) * zoom);
		} else {
			float flood =  [[properties objectForKey:CBFloodBeamAngleY] floatValue];
			float spot  =  [[properties objectForKey:CBSpotBeamAngleY] floatValue];
			return spot + ((flood-spot) * zoom);
		}
	} else {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			return [[properties objectForKey:CBBeamAngle] floatValue];
		} else {
			return [[properties objectForKey:CBBeamAngleY] floatValue];
		}
	}
}

- (NSMutableDictionary*)properties {
	return properties;
}

- (void)setProperties:(NSDictionary*)aValue {
	NSMutableDictionary* oldProperties = properties;
	properties = [[NSMutableDictionary dictionaryWithDictionary:aValue] retain];
	[oldProperties release];
}

- (NSMutableDictionary*)beamAngles {
	NSMutableDictionary* angles = [NSMutableDictionary dictionary];
	if ([self isZoomable]) {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			//Zoomable Circular
			[angles setObject:IIF([properties objectForKey:CBFloodBeamAngle], [NSNumber numberWithFloat:0.0]) forKey:CBFloodBeamAngle];
			[angles setObject:IIF([properties objectForKey:CBFloodFieldAngle], [NSNumber numberWithFloat:0.0]) forKey:CBFloodFieldAngle];
			[angles setObject:IIF([properties objectForKey:CBSpotBeamAngle], [NSNumber numberWithFloat:0.0]) forKey:CBSpotBeamAngle];
			[angles setObject:IIF([properties objectForKey:CBSpotFieldAngle], [NSNumber numberWithFloat:0.0]) forKey:CBSpotFieldAngle];			
		} else {
			//Zoomable Elliptical
			[angles setObject:IIF([properties objectForKey:CBFloodBeamAngleX], [NSNumber numberWithFloat:0.0]) forKey:CBFloodBeamAngleX];
			[angles setObject:IIF([properties objectForKey:CBFloodFieldAngleX], [NSNumber numberWithFloat:0.0]) forKey:CBFloodFieldAngleX];
			[angles setObject:IIF([properties objectForKey:CBSpotBeamAngleX], [NSNumber numberWithFloat:0.0]) forKey:CBSpotBeamAngleX];
			[angles setObject:IIF([properties objectForKey:CBSpotFieldAngleX], [NSNumber numberWithFloat:0.0]) forKey:CBSpotFieldAngleX];			
			[angles setObject:IIF([properties objectForKey:CBFloodBeamAngleY], [NSNumber numberWithFloat:0.0]) forKey:CBFloodBeamAngleY];
			[angles setObject:IIF([properties objectForKey:CBFloodFieldAngleY], [NSNumber numberWithFloat:0.0]) forKey:CBFloodFieldAngleY];
			[angles setObject:IIF([properties objectForKey:CBSpotBeamAngleY], [NSNumber numberWithFloat:0.0]) forKey:CBSpotBeamAngleY];
			[angles setObject:IIF([properties objectForKey:CBSpotFieldAngleY], [NSNumber numberWithFloat:0.0]) forKey:CBSpotFieldAngleY];			
		}
	} else {
		if ([[self beamShape] caseInsensitiveCompare:CBCircularBeam] == NSOrderedSame) {
			//Non-zoomable Circular
			[angles setObject:IIF([properties objectForKey:CBBeamAngle], [NSNumber numberWithFloat:0.0]) forKey:CBBeamAngle];
			[angles setObject:IIF([properties objectForKey:CBFieldAngle], [NSNumber numberWithFloat:0.0]) forKey:CBFieldAngle];			
		} else {
			//Non-zoomable Elliptical
			[angles setObject:IIF([properties objectForKey:CBBeamAngleX], [NSNumber numberWithFloat:0.0]) forKey:CBBeamAngleX];
			[angles setObject:IIF([properties objectForKey:CBFieldAngleX], [NSNumber numberWithFloat:0.0]) forKey:CBFieldAngleX];						
			[angles setObject:IIF([properties objectForKey:CBBeamAngleY], [NSNumber numberWithFloat:0.0]) forKey:CBBeamAngleY];
			[angles setObject:IIF([properties objectForKey:CBFieldAngleY], [NSNumber numberWithFloat:0.0]) forKey:CBFieldAngleY];						
		}
	}
	return angles;
}

- (void)setBeamAngles:(NSDictionary*)newAngles {
	NSEnumerator* enumerator = [[newAngles allKeys] objectEnumerator];
	id key;
	while (key = [enumerator nextObject]) {
		[properties setObject:[newAngles objectForKey:key] forKey:key];
	}
	[self initializeCurves];
}

- (void)setIsZoomable:(BOOL)zoomable {
	[self willChangeValueForKey:@"beamAngles"];
	[self willChangeValueForKey:@"beamAngleX"];
	[self willChangeValueForKey:@"beamAngleY"];
	[self willChangeValueForKey:@"fieldAngleX"];
	[self willChangeValueForKey:@"fieldAngleY"];
	[properties setObject:[NSNumber numberWithBool:zoomable] forKey:@"isZoomable"];
	[self didChangeValueForKey:@"beamAngleX"];
	[self didChangeValueForKey:@"beamAngleY"];
	[self didChangeValueForKey:@"fieldAngleX"];
	[self didChangeValueForKey:@"fieldAngleY"];	
	[self didChangeValueForKey:@"beamAngles"];
	[self initializeCurves];
}


- (float)zoom {
	return zoom;
}

- (void)setZoom:(float)newZoom {
	[self willChangeValueForKey:@"beamAngleX"];
	[self willChangeValueForKey:@"beamAngleY"];
	[self willChangeValueForKey:@"fieldAngleX"];
	[self willChangeValueForKey:@"fieldAngleY"];
	zoom = newZoom;
	[self initializeCurves];
	[self didChangeValueForKey:@"beamAngleX"];
	[self didChangeValueForKey:@"beamAngleY"];
	[self didChangeValueForKey:@"fieldAngleX"];
	[self didChangeValueForKey:@"fieldAngleY"];
}

- (void)encodeWithCoder:(NSCoder *)coder {
	[coder encodeObject:properties];
}

- (id)initWithCoder:(NSCoder *)coder {
	if (self = [super init]) {
		intensityCurveX = [[CBIntensityCurve alloc] init];
		intensityCurveY = [[CBIntensityCurve alloc] init];
		properties = [[coder decodeObject] retain];
		[self initializeCurves];
	}
	return self;	
}

- (void)makeReadOnly {
	[self willChangeValueForKey:@"properties"];
	NSDictionary* newProperties = [[NSDictionary dictionaryWithDictionary:properties] retain];
	[properties release];
	properties = newProperties;
	[self didChangeValueForKey:@"properties"];
}

- (void)dealloc {
	[properties release];
	[intensityCurveX release];
	[intensityCurveY release];
	[super dealloc];
}

@end