//
//  CBGeometricModel.m
//  CocoaBeam
//
//  Created by Jake on 3/23/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "CBGeometricModel.h"

@implementation CBGeometricModel

+ (void) initialize {
    NSArray *keys;
    keys = [NSArray arrayWithObjects: @"trimHeight", @"floorDistance", @"focusHeight", @"desiredArea", nil];

    [CBGeometricModel setKeys: keys
        triggerChangeNotificationsForDependentKey: @"geometryChanged"];

} 

- (BOOL) geometryChanged {
    return (YES);
}

- (id)init {
	if (self = [super init]){
		trimHeight=20.0;
		floorDistance=20.0;
		focusHeight=5.0;
		desiredArea = NSMakeSize(5.0,5.0);
	}
	return self;
}

- (float)trimHeight {
	return trimHeight;
}

- (void)setTrimHeight:(float)newTrimHeight {
	[self willChangeValueForKey:@"angleOfIncidence"];
	[self willChangeValueForKey:@"throwDistance"];
	trimHeight = newTrimHeight;
	[self didChangeValueForKey:@"throwDistance"];
	[self didChangeValueForKey:@"angleOfIncidence"];	
}

- (float)floorDistance {
	return floorDistance;
}

- (void)setFloorDistance:(float)newFloorDistance {
	floorDistance = newFloorDistance;
}

- (float)focusHeight {
	return focusHeight;
}

- (void)setFocusHeight:(float)newFocusHeight
{
	[self willChangeValueForKey:@"angleOfIncidence"];
	[self willChangeValueForKey:@"throwDistance"];
	focusHeight = newFocusHeight;
	[self didChangeValueForKey:@"throwDistance"];
	[self didChangeValueForKey:@"angleOfIncidence"];
}

- (float)angleOfIncidence { 
	if (trimHeight==0) return FP_NAN;
	float val = DEGREES(atan(floorDistance/(trimHeight-focusHeight)));
	if (val<0) val = 180+val;
	return val;
}

- (float)throwDistance {
	float height = (trimHeight-focusHeight);
	return sqrt((height*height) + (floorDistance * floorDistance));
}

- (float)floorPointForAngleOffset:(float)angle {
	return floorDistance-(trimHeight*tan(RADIANS(([self angleOfIncidence] + angle))));
}

- (float)floorWidthForAngle:(float)angle {
	float throwToFloor = [self throwDistance] + ([self focusHeight]/cos(RADIANS([self angleOfIncidence])));
	return 2 * throwToFloor * tan(RADIANS(angle/2));
	//return 2 * [self throwDistance] * tan(RADIANS(angle/2));
}

- (NSSize)desiredArea {
	return desiredArea;
}

- (void)setDesiredArea:(NSSize)newDesiredArea {
	[self willChangeValueForKey:@"desiredAreaWidth"];
	[self willChangeValueForKey:@"desiredAreaHeight"];
	desiredArea = newDesiredArea;
	[self didChangeValueForKey:@"desiredAreaWidth"];
	[self didChangeValueForKey:@"desiredAreaHeight"];
}

- (float)desiredAreaWidth {
	return desiredArea.width;
}

- (void)setDesiredAreaWidth:(float)newDesiredAreaWidth {
	[self willChangeValueForKey:@"desiredArea"];
	desiredArea.width = newDesiredAreaWidth;
	[self didChangeValueForKey:@"desiredArea"];	
}

- (float)desiredAreaHeight {
	return desiredArea.height;
}

- (void)setDesiredAreaHeight:(float)newDesiredAreaHeight {
	[self willChangeValueForKey:@"desiredArea"];
	desiredArea.height = newDesiredAreaHeight;
	[self didChangeValueForKey:@"desiredArea"];
}

@end
