//
//  CBIntensityCurve.h
//  CocoaBeam
//
//  Created by Jake on 3/21/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "CBIntensityCurve.h"

int pointSortX(id value1, id value2, void *context) {
    NSPoint p1 = [value1 pointValue];
    NSPoint p2 = [value2 pointValue];

    if (p1.x < p2.x)
        return NSOrderedAscending;
    else if (p1.x > p2.x)
        return NSOrderedDescending;
    else
        return NSOrderedSame;
}

@implementation CBIntensityCurve

- (NSArray*)points {
	return points;
}

- (void)setPoints:(NSArray*)newPoints {
	NSArray* oldPoints = points;
	points = [[newPoints sortedArrayUsingFunction:pointSortX context:NULL] retain];
	[oldPoints release];
	
	float x1 = [[points objectAtIndex:0] pointValue].x;
	float x2 = [[points objectAtIndex:1] pointValue].x;
	float x3 = [[points objectAtIndex:2] pointValue].x;
	
	float y1 = [[points objectAtIndex:0] pointValue].y;
	float y2 = [[points objectAtIndex:1] pointValue].y;
	float y3 = [[points objectAtIndex:2] pointValue].y;
	
	A = (y3-y2)/((x3-x2)*(x3-x1)) - (y1-y2)/((x1-x2)*(x3-x1));
	B = (y1 - y2 + (A*((x2*x2)-(x1*x1)))) / (x1-x2);
	C = y1 - (A*(x1*x1)) - (B*x1);
	
	float abscissa = -B/(2*A);
	if (abscissa < 0 || abscissa > (-B - sqrt((B*B)-(4*A*C))) / (2*A)) {
		type = CBCurveParabolic;
	} else {
		type = CBCurveLinear;
	}
}

/*- (float)valueForX:(float)x {
	switch (type) {
		case CBCurveParabolic:
			return A*(x*x) + B*x + C;
		
		case CBCurveLinear:
			break;
	}
	return 0;
}*/

- (float)angleForIntensity:(float)intensity {
	switch (type) {
		case CBCurveParabolic:
			return (-B - sqrt((B*B)-(4*A*(C-intensity)))) / (2*A);
		
		default:
		case CBCurveLinear:
			{
		    NSPoint point = [[points objectAtIndex:1] pointValue];
			if (intensity < point.y) {
				NSPoint nextPoint = [[points objectAtIndex:2] pointValue];
				float m = (nextPoint.y-point.y)/(nextPoint.x - point.x);
				float b = point.y - (m*point.x);
				return (intensity - b)/m;
			} else if (intensity > point.y) {
				NSPoint prevPoint = [[points objectAtIndex:0] pointValue];
				float m = (point.y - prevPoint.y)/(point.x - prevPoint.x);
				float b = point.y - (m*point.x);
				return (intensity - b)/m;
			} else
				return point.x;
				break;
			}
	}
}

/*- (float)xIntercept {
	switch (type) {
		case CBCurveParabolic:
			return (-B - sqrt((B*B)-(4*A*C))) / (2*A);
			
		case CBCurveLinear:
			break;
	}
}*/

@end
